import { EasingType, EntityInventoryComponent, EntityRideableComponent, EntityRidingComponent, ItemStack, system, world } from "@minecraft/server";
import { decreaseItem } from "lib/item";
const dimensions = ["overworld", "nether", "the_end"].map(name => world.getDimension(name));
const FUEL_SLOTS = [0, 1, 2];
const UPGRADE_SLOTS = [3, 4, 5];
const PROGRESS_BAR_SLOT = 12;
const FRAMES = 12;
const vehicleName = {
    "horizon_engineers:plane": "§p§l§a§n§e§r"
};
const fuelData = [
    { id: "minecraft:coal", fuelDuration: 6 },
    { id: "minecraft:charcoal", fuelDuration: 6 },
    { id: "minecraft:coal_block", fuelDuration: 48 },
];
const dummyItem = {
    id: "horizon_engineers:progress_bar",
    name: "none"
};
world.afterEvents.entitySpawn.subscribe(({ entity }) => {
    if (entity.typeId === "minecraft:item") {
        const item = entity.getComponent("item")?.itemStack;
        if (!item || item.typeId !== dummyItem.id)
            return;
        entity.remove();
    }
    if (vehicleName[entity.typeId]) {
        const inv = entity.getComponent(EntityInventoryComponent.componentId).container;
        const itemProgressBar = new ItemStack(dummyItem.id);
        itemProgressBar.nameTag = dummyItem.name;
        entity.setDynamicProperty("horizon_engineers:movement", "ground");
        inv.setItem(PROGRESS_BAR_SLOT, itemProgressBar);
        entity.nameTag = vehicleName[entity.typeId];
    }
});
system.runInterval(() => {
    for (const player of world.getAllPlayers()) {
        const riding = player.getComponent(EntityRidingComponent.componentId);
        const plane = riding?.entityRidingOn;
        const isDriving = player.getDynamicProperty("horizon_engineers:is_driving");
        if (!riding && isDriving) {
            player.camera.clear();
            player.setDynamicProperty("horizon_engineers:is_driving", false);
        }
        if (!plane || plane.typeId !== "horizon_engineers:plane")
            continue;
        if (!isDriving && riding)
            player.setDynamicProperty("horizon_engineers:is_driving", true);
        const movement = plane.getDynamicProperty("horizon_engineers:movement"); //ground, air
        if (!plane.isOnGround) {
            const planeInv = plane.getComponent(EntityInventoryComponent.componentId).container;
            const coalUpgrade = searchItem(planeInv, UPGRADE_SLOTS, "horizon_engineers:coal_upgrade");
            const speedUpgrade = searchItem(planeInv, UPGRADE_SLOTS, "horizon_engineers:speed_upgrade");
            if (movement === "ground")
                plane.setDynamicProperty("horizon_engineers:movement", "air");
            flySystem(plane, player, speedUpgrade ? 0.95 : 0.7);
            setCamera(player, { y: 3.5, xz: 4.9 });
        }
        else if (plane.isOnGround && movement === "air") {
            player.camera.clear();
            plane.setDynamicProperty("horizon_engineers:movement", "ground");
        }
    }
});
system.runInterval(() => {
    const planeEntities = dimensions.flatMap(dim => dim.getEntities({ type: "horizon_engineers:plane" }));
    for (const plane of planeEntities) {
        const inv = plane.getComponent(EntityInventoryComponent.componentId).container;
        const rideable = plane.getComponent(EntityRideableComponent.componentId);
        const mainRider = rideable.getRiders()[0];
        // console.warn(mainRider)
        if (!mainRider && !plane.isOnGround && !plane.isInWater) {
            const viewDir = plane.getViewDirection();
            plane.addEffect("slow_falling", 1, { amplifier: 1, showParticles: false });
            plane.applyImpulse({ x: viewDir.x * 0.05, y: 0, z: viewDir.z * 0.05 });
        }
        const hasFuel = plane.getProperty("horizon_engineers:has_fuel");
        const fuel = plane.getDynamicProperty("horizon_engineers:fuel") || 0;
        const fuelId = plane.getDynamicProperty("horizon_engineers:fuel_id");
        // if (!mainRider && fuel === 0) return
        if (fuel === 0 && hasFuel) {
            plane.triggerEvent("horizon_engineers:no_fuel");
        }
        if (fuel === 0 && !hasFuel) {
            const getfuelItem = searchFuel(inv, FUEL_SLOTS, fuelData);
            if (!getfuelItem)
                return;
            const { id, fuelDuration, slot } = getfuelItem;
            const item = inv.getItem(slot);
            const mainSlot = FUEL_SLOTS[0];
            inv.setItem(slot, decreaseItem(item));
            if (slot !== mainSlot)
                inv.swapItems(slot, mainSlot, inv);
            updateProgressBar(plane, PROGRESS_BAR_SLOT, FRAMES);
            plane.setDynamicProperty("horizon_engineers:fuel", fuelDuration * 20);
            plane.setDynamicProperty("horizon_engineers:fuel_id", id);
            plane.setDynamicProperty("horizon_engineers:frame", FRAMES);
            plane.triggerEvent("horizon_engineers:fuel");
        }
        else if (fuel > 0 && hasFuel) {
            const frameProperty = plane.getDynamicProperty("horizon_engineers:frame");
            const fuelDuration = getFuelDuration(fuelId, fuelData);
            const newFuel = fuel - 1;
            const result = Math.floor((fuelDuration / FRAMES) * frameProperty);
            plane.setDynamicProperty("horizon_engineers:fuel", newFuel);
            if (newFuel === 0) {
                plane.setDynamicProperty("horizon_engineers:fuel_id", "none");
                updateProgressBar(plane, PROGRESS_BAR_SLOT, undefined);
            }
            else if (result >= newFuel) {
                updateProgressBar(plane, PROGRESS_BAR_SLOT, frameProperty - 1);
            }
        }
    }
});
function getFuelDuration(fuelId, fuelData) {
    const fuelDuration = fuelData.find(f => f.id === fuelId).fuelDuration * 20;
    return fuelDuration;
}
function searchItem(inv, slots, id) {
    let hasItem = false;
    for (let i = 0; i < slots.length; i++) {
        const slot = slots[i];
        const item = inv.getItem(slot);
        if (!item || item.typeId !== id)
            continue;
        hasItem = true;
        break;
    }
    return hasItem;
}
function searchFuel(inv, fuelSlots, fuelData) {
    let fuelItem = undefined;
    for (let i = 0; i < fuelSlots.length; i++) {
        const slot = fuelSlots[i];
        const item = inv.getItem(slot);
        if (!item)
            continue;
        fuelItem = fuelData.find(f => f.id === item?.typeId);
        if (fuelItem)
            return { ...fuelItem, slot: i };
    }
    return undefined;
}
function updateProgressBar(entity, slot, progress) {
    const inv = entity.getComponent(EntityInventoryComponent.componentId).container;
    const itemProgressBar = inv.getItem(slot);
    progress = progress === undefined ? 1000 : progress;
    itemProgressBar.nameTag = progress.toString();
    entity.setDynamicProperty("horizon_engineers:frame", progress);
    inv.setItem(slot, itemProgressBar);
}
function flySystem(plane, player, speedMult) {
    const viewDir = player.getViewDirection();
    plane.clearVelocity();
    plane.applyImpulse({
        x: plane.getViewDirection().x * speedMult,
        y: viewDir.y * 0.5,
        z: plane.getViewDirection().z * speedMult
    });
}
function setCamera(player, distance) {
    const viewDir = player.getViewDirection();
    const headLoc = player.getHeadLocation();
    const rot = player.getRotation();
    const cameraLocation = {
        x: headLoc.x + (-viewDir.x * distance.xz),
        y: (headLoc.y + (rot.x < 0 ? 3 : 1)) + ((-viewDir.y * distance.y) * 1.0),
        z: headLoc.z + (-viewDir.z * distance.xz)
    };
    const facing = {
        x: headLoc.x + (viewDir.x * distance.xz),
        y: (headLoc.y + 1) + ((viewDir.y * distance.y) * 1.0),
        z: headLoc.z + (viewDir.z * distance.xz)
    };
    player.camera.setCamera("minecraft:free", { facingLocation: facing, location: cameraLocation, easeOptions: { easeTime: 0.5, easeType: EasingType.Linear } });
}
